"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.postAnimatedWarns = postAnimatedWarns;
exports.validateHyUI = validateHyUI;
exports.validateAnimatedValue = validateAnimatedValue;
exports.validateVersionList = exports.getVersionFromStr = exports.minValidVersion = void 0;
var _childProcess = require("child_process");
var _semver = _interopRequireDefault(require("semver"));
var _axios = _interopRequireDefault(require("axios"));
var _os = require("os");
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
var minValidVersion = "1.3.29";
exports.minValidVersion = minValidVersion;
var excludeList = [
    "**/node_modules/react-native/*",
    "**/node_modules/react-native-web/*",
    "**/node_modules/@hyext/hy-ui/*", 
];
function postAnimatedWarns(files, extUuid) {
    var content = "".concat(extUuid, " \u4F7F\u7528\u4E86 new Animated.Value, \u6587\u4EF6\u5217\u8868: \n-").concat(files.join("\n-"));
    if (!process.env.RN_ANIMATED_URL) return;
    try {
        (0, _axios).default({
            method: "post",
            url: process.env.RN_ANIMATED_URL,
            data: {
                msgtype: "text",
                text: {
                    content: content
                }
            }
        });
    } catch (err) {
        console.warn("\u544A\u8B66\u53D1\u9001\u5931\u8D25");
    }
}
var getVersionFromStr = function(content) {
    var pkgName = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : "@hyext/hy-ui";
    var ref, ref1;
    var reg = new RegExp("".concat(pkgName, "@(.+)$"), "i");
    var lines = (ref1 = (ref = content === null || content === void 0 ? void 0 : content.split(_os.EOL)) === null || ref === void 0 ? void 0 : ref.map(function(xs) {
        var ref;
        return (ref = reg.exec(xs)) === null || ref === void 0 ? void 0 : ref[1];
    })) === null || ref1 === void 0 ? void 0 : ref1.filter(function(xs) {
        return !!xs;
    });
    return lines;
};
exports.getVersionFromStr = getVersionFromStr;
var validateVersionList = function(list) {
    var baseVersion = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : minValidVersion;
    return list.every(function(xs) {
        return _semver.default.satisfies(xs, ">=".concat(baseVersion), {
            includePrerelease: true
        });
    });
};
exports.validateVersionList = validateVersionList;
function validateHyUI(projectRoot) {
    var baseVersion = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : minValidVersion;
    var pkgName = "@hyext/hy-ui";
    //   ~/Downloads took 4s
    // ❯ npm --prefix ~/Developer/huya/hyext/just-dance ls @hyext/hy-ui --parseable --long
    // /Users/simon/Developer/huya/hyext/just-dance/node_modules/@hyext/hy-ui:@hyext/hy-ui@1.3.26
    var ref = (0, _childProcess).spawnSync("npm", [
        "ls",
        pkgName,
        "--parseable",
        "--long", 
    ], {
        cwd: projectRoot
    }), stdout = ref.stdout, stderr = ref.stderr, status = ref.status, error = ref.error;
    if (error) return true;
    return validateVersionList(getVersionFromStr(stdout === null || stdout === void 0 ? void 0 : stdout.toString()), baseVersion);
}
function validateAnimatedValue(projectRoot, extUuid) {
    var exclude = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : excludeList;
    var ref, ref1;
    // rg -uuuu -l 'new Animated.Value' ~/Developer/huya/hyext/just-dance --type js -g '!{**/node_modules/react-native/*,**/node_modules/react-native-web/*,**/node_modules/@hyext/hy-ui/*}'
    var ref2 = (0, _childProcess).spawnSync("rg", [
        "-uuuuu",
        "--files-with-matches",
        "new Animated.Value",
        projectRoot,
        "--type",
        "js",
        "-g",
        "!{".concat(exclude.join(","), "}"), 
    ], {
        cwd: projectRoot
    }), stdout = ref2.stdout, status = ref2.status, error = ref2.error;
    if (error) {
        console.warn("validateAnimatedValue error", error.message);
    }
    var files = (ref1 = (ref = stdout === null || stdout === void 0 ? void 0 : stdout.toString()) === null || ref === void 0 ? void 0 : ref.split(_os.EOL)) === null || ref1 === void 0 ? void 0 : ref1.filter(function(xs) {
        return !!xs;
    });
    if (files.length > 0) {
        postAnimatedWarns(files.map(function(xs) {
            return xs.replace("".concat(projectRoot, "/"), "");
        }), extUuid);
    }
}
