"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.defaultFactory = defaultFactory;
exports.createModuleIdHOF = createModuleIdHOF;
var _path = _interopRequireDefault(require("path"));
var _ramda = require("ramda");
function _arrayLikeToArray(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for(var i = 0, arr2 = new Array(len); i < len; i++)arr2[i] = arr[i];
    return arr2;
}
function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) return arr;
}
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _iterableToArrayLimit(arr, i) {
    var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"];
    if (_i == null) return;
    var _arr = [];
    var _n = true;
    var _d = false;
    var _s, _e;
    try {
        for(_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true){
            _arr.push(_s.value);
            if (i && _arr.length === i) break;
        }
    } catch (err) {
        _d = true;
        _e = err;
    } finally{
        try {
            if (!_n && _i["return"] != null) _i["return"]();
        } finally{
            if (_d) throw _e;
        }
    }
    return _arr;
}
function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _slicedToArray(arr, i) {
    return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}
function _unsupportedIterableToArray(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(n);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}
var pathSep = _path.default.sep;
function defaultFactory() {
    var fileToIdMap = new Map();
    var nextId = 0;
    return function(modulePath) {
        var id = fileToIdMap.get(modulePath);
        if (typeof id !== "number") {
            id = nextId++;
            fileToIdMap.set(modulePath, id);
        }
        return id;
    };
}
var caches = new Map();
var getId = function(modulePath, options) {
    var projectRoot = options.projectRoot, SDKAlias = options.SDKAlias;
    var genSDKReg = function(SDKNames) {
        var uniformNames = SDKNames.map(function(x) {
            return x.replace("/", "_");
        });
        return new RegExp("node_modules_(".concat(uniformNames.join("|"), ")"), "ig");
    };
    //RN JS 全部以 RN_ 开头
    if (modulePath.includes("node_modules" + pathSep + "react-native" + pathSep + "Libraries" + pathSep)) {
        return "RN_".concat(_path.default.basename(modulePath, ".js"));
    }
    // 如果模块不在 projectRoot 内, 比如 metro-runtime/src/polyfills/requires.js
    // id 变成了 _Users_simon_Developer_huya_hyext_node-pack_node_modules_metro-runtime_src_polyfills_require
    // 为了能正确过滤basebundle中已有的模块(node_modules_metro-runtime_src_polyfills_require), 需要进行处理
    var name = [
        projectRoot,
        process.env.MATRIX_SHARE_PROJECT_RN_ROOT,
        process.env.MATRIX_PACKER_ROOT
    ].filter(Boolean).reduce(function(acc, cur) {
        return acc.replace("".concat(cur, "/"), "");
    }, modulePath);
    // remove .js extendtion
    name = name.replace(/.js$/, "");
    // 路径分隔符换成 _
    var regExp = pathSep == "\\" ? new RegExp("\\\\", "g") : new RegExp(pathSep, "g");
    name = name.replace(regExp, "_");
    // 把业务/base jsbundle 里的非 node_modules 里的模块 id 都带上根目录名
    if (modulePath.includes(projectRoot) && !modulePath.includes("node_modules")) {
        name = _path.default.basename(projectRoot) + "_" + name;
    }
    Object.entries(SDKAlias).forEach(function(param) {
        var _param = _slicedToArray(param, 2), alias = _param[0], SDKNames = _param[1];
        if (SDKNames.length <= 0) return;
        var SDKReg = genSDKReg(SDKNames);
        if (SDKReg.test(name)) {
            name = name.replace(SDKReg, alias);
        }
    });
    return name;
};
function createModuleIdHOF(opts) {
    return function createModuleIdFactory() {
        return function(modulePath) {
            var cacheId = caches.get(modulePath);
            if (cacheId) return cacheId;
            var moduleId = getId(modulePath, (0, _ramda).omit([
                "filterModules"
            ], opts));
            // 如果是非baseBundle中的包, 那么加上一些额外的字符来避免以下情况:
            // 1. baseBundle加了一些包, 但是没有在同步到模块清单中
            // 2. 如果baseBundle/busiBundle 都用同一个模块名称, 那么可能会导致"重复定义"的错误
            // 所以, 非baseBundle模块清单中的模块, 在命名上做一些个性化的区分
            if (opts.filterModules.size > 0 && !opts.filterModules.has(moduleId)) {
                moduleId = "".concat(moduleId, "_#biz");
            }
            caches.set(modulePath, moduleId);
            return moduleId;
        };
    };
}
