"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = matrixServer;
var _regeneratorRuntime = _interopRequireDefault(require("regenerator-runtime"));
var _metro = _interopRequireDefault(require("metro"));
var _metroCore = require("metro-core");
var _metroConfig = require("metro-config");
var _path = _interopRequireDefault(require("path"));
var _cliServerApi = require("@react-native-community/cli-server-api");
var _watchMode = _interopRequireDefault(require("./watchMode"));
var _constants = require("../constants");
var _hyTerminalReporter = _interopRequireDefault(require("../lib/HyTerminalReporter"));
var _hyExtMiddleware = require("./hyExtMiddleware");
var _getExtMetroConfig = require("../matrix-config/getExtMetroConfig");
function matrixServer(config) {
    return _matrixServer.apply(this, arguments);
}
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _asyncToGenerator(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _matrixServer() {
    _matrixServer = _asyncToGenerator(_regeneratorRuntime.default.mark(function _callee(config) {
        var reportEvent, terminal, ReporterImpl, terminalReporter, reporter, defaultConfig, finalConfig, ref, middleware, websocketEndpoints, messageSocketEndpoint, eventsSocketEndpoint, customEnhanceMiddleware, serverInstance;
        return _regeneratorRuntime.default.wrap(function _callee$(_ctx) {
            while(1)switch(_ctx.prev = _ctx.next){
                case 0:
                    ;
                    terminal = new _metroCore.Terminal(process.stdout);
                    ReporterImpl = getReporterImpl(config.customLogReporterPath);
                    terminalReporter = new ReporterImpl(terminal);
                    reporter = {
                        update: function update(event) {
                            terminalReporter.update(event);
                            if (reportEvent) {
                                reportEvent(event);
                            }
                        }
                    };
                    defaultConfig = (0, _getExtMetroConfig).getExtMetroConfig({
                        projectRoot: config.projectRoot,
                        mode: "dev",
                        projectConfig: config.projectConfig,
                        extMetroConfig: config.extMetroConfig
                    });
                    finalConfig = (0, _metroConfig).mergeConfig(defaultConfig, {
                        reporter: reporter,
                        server: {
                            port: config.port || 8081
                        }
                    });
                    ref = (0, _cliServerApi).createDevServerMiddleware({
                        host: config.host || _constants.DEFAULT_HOST,
                        port: finalConfig.server.port,
                        watchFolders: finalConfig.watchFolders
                    }), middleware = ref.middleware, websocketEndpoints = ref.websocketEndpoints, messageSocketEndpoint = ref.messageSocketEndpoint, eventsSocketEndpoint = ref.eventsSocketEndpoint;
                    middleware.use(_cliServerApi.indexPageMiddleware);
                    customEnhanceMiddleware = finalConfig.server.enhanceMiddleware;
                    finalConfig.server.enhanceMiddleware = function(metroMiddleware, server) {
                        if (customEnhanceMiddleware) {
                            metroMiddleware = customEnhanceMiddleware(metroMiddleware, server);
                        }
                        // 请保持 middleware 在 metroMiddleware 之前执行
                        // 否则 metro 处理 /symbolicate 请求时会得到空的body
                        // https://github.com/facebook/metro/issues/660
                        return _hyExtMiddleware.hyExtMiddleware.use(middleware).use(metroMiddleware);
                    };
                    _ctx.next = 13;
                    return _metro.default.runServer(finalConfig, {
                        host: config.host || _constants.DEFAULT_HOST,
                        onReady: config.onReady,
                        onError: config.onError,
                        // @ts-ignore @types/metro 太旧了
                        hmrEnabled: true,
                        // @ts-ignore @types/metro 太旧了
                        websocketEndpoints: websocketEndpoints
                    });
                case 13:
                    serverInstance = _ctx.sent;
                    reportEvent = eventsSocketEndpoint.reportEvent;
                    if (config.interactive !== false) {
                        (0, _watchMode).default(messageSocketEndpoint);
                    }
                    serverInstance.keepAliveTimeout = 30000;
                    return _ctx.abrupt("return", serverInstance);
                case 18:
                case "end":
                    return _ctx.stop();
            }
        }, _callee);
    }));
    return _matrixServer.apply(this, arguments);
}
function getReporterImpl(customLogReporterPath) {
    // console.log('###############', customLogReporterPath)
    if (customLogReporterPath === undefined) {
        return _hyTerminalReporter.default;
    }
    try {
        return require(customLogReporterPath);
    } catch (e) {
        if (e.code !== "MODULE_NOT_FOUND") {
            throw e;
        }
        return require(_path.default.resolve(customLogReporterPath));
    }
}
