"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = void 0;
var _readline = _interopRequireDefault(require("readline"));
var _hooksStdout = _interopRequireDefault(require("./hooksStdout"));
var _logger = _interopRequireDefault(require("../lib/logger"));
var _constants = require("../constants");
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function printWatchModeInstructions() {
    _logger.default.log('\n\n\u6309 "r" \u952E\u91CD\u65B0\u52A0\u8F7D\u5C0F\u7A0B\u5E8F.\n\u6309 "d" \u952E\u6253\u5F00\u5C0F\u7A0B\u5E8F\u7684\u5F00\u53D1\u83DC\u5355');
}
function enableWatchMode(messageSocket) {
    var printInstructions = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : true;
    // We need to set this to true to catch key presses individually.
    // As a result we have to implement our own method for exiting
    // and other commands (e.g. ctrl+c & ctrl+z)
    if (!process.stdin.setRawMode) {
        _logger.default.debug("Watch mode is not supported in this environment");
        return;
    }
    _readline.default.emitKeypressEvents(process.stdin);
    process.stdin.setRawMode(true);
    // We have no way of knowing when the dependency graph is done loading
    // except by hooking into stdout itself. We want to print instructions
    // right after its done loading.
    var restore = (0, _hooksStdout).default(function(output) {
        if (output.includes(_constants.MATRIX_READY_MESSAGE)) {
            if (printInstructions) {
                printWatchModeInstructions();
            }
            restore();
        }
    });
    process.stdin.on("keypress", function(_key, data) {
        var ctrl = data.ctrl, name = data.name;
        if (ctrl === true) {
            switch(name){
                case "c":
                    process.exit();
                    break;
                case "z":
                    process.emit("SIGTSTP", "SIGTSTP");
                    break;
            }
        } else if (name === "r") {
            messageSocket.broadcast("reload", null);
            _logger.default.info("\u91CD\u65B0\u52A0\u8F7D\u5C0F\u7A0B\u5E8F...");
        } else if (name === "d") {
            messageSocket.broadcast("devMenu", null);
            _logger.default.info("\u6253\u5F00\u5C0F\u7A0B\u5E8F\u5F00\u53D1\u83DC\u5355...");
        }
    });
}
var _default = enableWatchMode;
exports.default = _default;
